package Badger::Data::Types;

use Badger::Factory::Class
    version   => 0.01,
    debug     => 0,
    item      => 'type',
    path      => 'Badger::Data::Type  BadgerX::Data::Type';


1;

__END__

=head1 NAME

Badger::Data::Types - factory module for Badger::Data::Type objects

=head1 AUTHOR

Andy Wardley L<http://wardley.org/>

